package MayLib;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JDialog;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;

public abstract class AbstractDialog extends JDialog
{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Art des Meneintrags
	 * 
	 */
	public static enum MenuItemType
	{
		ITEM_PLAIN, ITEM_CHECK, ITEM_RADIO
	}

	public Component owner;

	/**
	 * <li><b><i>showDialog</i></b>
	 * <br>
	 * <br> 
	 * public void showDialog()
	 * <br>
	 * <br>
	 * Macht die Komponente sichtbar.
	 * 
	 */
	public void showDialog()
	{
		initDialog();
		this.setModalityType(DEFAULT_MODALITY_TYPE);
		centerWindow();
		this.setVisible(true);
		}

	/**
	 * <li><b><i>showDialog</i></b>
	 * <br>
	 * <br> 
	 * public void showDialog(Component owner)
	 * <br>
	 * <br>
	 * Macht die Komponente sichtbar.
	 * 
	 * @param owner
	 * 		- Eigentmer der Komponente
	 */
	public void showDialog(Component owner)
	{
		this.owner = owner;
		this.showDialog();
	}

	/**
	 * <li><b><i>centerWindow</i></b>
	 * <br>
	 * <br>
	 * public void centerWindow()
	 * <br>
	 * <br>
	 * Zentriert die Komponente entweder auf dem Bildschirm
	 * oder auf dem aufrufenden Fenster.
	 * 
	 */
	public void centerWindow()
	{
		this.setLocationRelativeTo(owner);
	}

	/**
	 * <li><b><i>createMenuItem</i></b>
	 * <br>
	 * <br>
	 * public JMenuItem createMenuItem(JMenu menu, String miName, MenuItemType miType, ActionListener actionListener,<br>&nbsp String sText, String imageFile, int acceleratorKey, String sToolTip)
	 * <br>
	 * <br>
	 * Erstellt einen Meneintrag.
	 * <br>
	 * <br>
	 * @param menu
	 * 	- Das Men, zu dem dieser Meneintrag gehrt.
	 * @param miName
	 * - Optionaler Name des Meneintrags oder <b>null</b>.
	 * @param miType
	 * 	- Der Typ des Meneintrags <b>MenuItemType</b>.
	 * @param actionListener
	 *  - Der ActionListener, der verwendet werden soll, wenn der Meneintrag ausgewhlt wurde oder <b>null</b>.
	 * @param sText
	 * 	- Der Text des Meneintrags.
	 * @param image
	 * 	- Symbol, welches links vor dem Text angezeigt werden soll oder <b>null</b>.
	 * @param acceleratorKey
	 * 	- Optionales Tastaturkrzel oder <b>0</b>.
	 * @param sToolTip
	 * 	- Optionaler Text fr den Tooltip oder <b>null</b>.
	 * @return
	 * 	  Meneintrag.
	 */
	public JMenuItem createMenuItem(JMenu menu, String miName, MenuItemType miType, ActionListener actionListener, String sText, ImageIcon image, int acceleratorKey,
			String sToolTip)
	{
		// MenuItem erstellen
		JMenuItem menuItem = new JMenuItem();

		switch (miType)
		{
		case ITEM_PLAIN:
			menuItem = new JMenuItem();
			break;

		case ITEM_RADIO:
			menuItem = new JRadioButtonMenuItem();
			break;

		case ITEM_CHECK:
			menuItem = new JCheckBoxMenuItem();
			break;
		}

		
		// Name des Menentrags
		menuItem.setName(miName);
		
		// Men Text hinzufgen
		menuItem.setText(sText);

		// Optionales Image hinzufgen
		menuItem.setIcon(image);
		
		// Optionales Tastaturkrzel hinzufgen
		menuItem.setMnemonic(acceleratorKey);

		// Optionalen Tooltip hinzufgen
		menuItem.setToolTipText(sToolTip);

		// Action Listener hinzufgen
		menuItem.addActionListener(actionListener);

		// Meneintrag zum Men hinzufgen
		menu.add(menuItem);

		// Rckgabe des Meneintrags
		return menuItem;
	}

	/**
	 * <li><b><i>createMenu</i></b>
	 * <br>
	 * <br>
	 * public JMenu createMenu(JMenuBar menuBar, String menuText, String menuName, int acceleratorKey)
	 * <br>
	 * <br>
	 * Erstellt einen Men.
	 * <br>
	 * <br>
	 * @param menuBar
	 * - Die Menleiste, zu dem dieses Men gehrt.
	 * @param menuText
	 * - Der Text des Mens.
	 * @param menuName
	 * - Optionaler Name des Mens oder <b>null</b>.
	 * @param acceleratorKey
	 * - Optionales Tastaturkrzel oder <b>0</b>.
	 * @return
	 * 	Men.
	 */
	public JMenu createMenu(JMenuBar menuBar, String menuText, String menuName, int acceleratorKey)
	{
		JMenu menu = null;

		// Hinzufgen des Mens
		menu = new JMenu();
		menu.setText(menuText);
		menu.setName(menuName);

		// Optionales Tastaturkrzel hinzufgen
		if (acceleratorKey > 0)
			menu.setMnemonic(acceleratorKey);

		menuBar.add(menu);

		return menu;
	}

	/**
	 * <li><b><i>createSubMenu</i></b>
	 * <br>
	 * <br>
	 * public JMenu createSubMenu(JMenu mainMenu, String menuText, String menuName, int acceleratorKey)
	 * <br>
	 * <br>
	 * Erstellt ein Untermen.
	 * <br>
	 * <br>
	 * @param mainMenu
	 * - Das Men, zu dem das Untermen hinzugefgt werden soll.
	 * @param menuText
	 * - Der Text des Mens.
	 * @param menuName
	 * - Optionaler Name des Untermens oder <b>null</b>.
	 * @param acceleratorKey
	 * - Optionales Tastaturkrzel oder <b>0</b>.
	 * @return
	 * - Untermen.
	 */
	public JMenu createSubMenue(JMenu mainMenu, String menuText, String menuName, int acceleratorKey)
	{
		JMenu menu = null;

		// Hinzufgen des Mens als Untermen des Hauptmens
		menu = new JMenu();
		menu.setText(menuText);
		menu.setName(menuName);

		// Optionales Tastaturkrzel hinzufgen
		if (acceleratorKey > 0)
			menu.setMnemonic(acceleratorKey);

		mainMenu.add(menu);

		return menu;
	}

	/**
	 * Methode zum Erstellen der Komponenten des Fensters.
	 */
	public abstract void initializeComponents();
	
	/**
	 * Methode zur Initialisierung der Steuerelemente bevor
	 * das Fenster angezeigt wird.
	 * Wird von der ffentlichen Methode <b>showDialog()</b> aufgerufen. 
	 */
	public abstract void initDialog();

	
	// method to maximize the window
	public void makeFrameFullSize()
	{
		// screenSize.height*0.93 to assure that the buttons at the bottom are visible
	  Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	  this.setBounds(0,0, (int)(screenSize.width *0.99), (int)(screenSize.height*0.94));
	}	

}
