package MayLib;

import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;


public class MayLibSearch extends JDialog implements ActionListener, WindowListener
{
	private static final long serialVersionUID = 1L;
	private static MayLibSearch myInstance;
	
	private Component owner;
	private JLabel lbl1;
	private JTextField	tfSuche;
	private JCheckBox checkGrossKlein;
	private JButton btnBeenden, btnWeitersuchen;
	
	private MayLib_Start  	Father;
	private Font Plainfont, Boldfont; 
	
	private JTextArea textArea;
	
	// die Position des gefundenen Textmusters
	private int	foundPos = -1;
		
	private boolean newDialog = true;
	
	private MayLibSearch()
	{
		initializeComponents();	
	}
	
	private MayLibSearch(JTextArea textArea, MayLib_Start Father)
	{
		this();
		this.Plainfont = Father.Plainfont;
		this.Boldfont = Father.Boldfont;
		this.setFont(Plainfont);
		this.Father = Father;
		
		//////
		this.textArea = textArea;
		
		// Den markierten Text aus der TextArea in das Suchfeld bernehmen
		// und dort ebenfalls markieren
		if (this.textArea.getSelectedText() != null)
		{
			tfSuche.setText(this.textArea.getSelectedText());
		
			foundPos = this.textArea.getSelectionStart();
			
			// Text im Suchfeld markieren
			tfSuche.selectAll();
		}
		
		// Durch den Fokusverlust ist der markierte Text im Hauptfenster
		// nicht mehr sichtbar. Durch das erneute setzen des Fokus
		// wird die Markierung wieder angezeigt.
		this.textArea.requestFocusInWindow();		
		
	}
	
	public static MayLibSearch getInstance(JTextArea textArea, MayLib_Start Father)
	{
		
		if (MayLibSearch.myInstance == null)
			MayLibSearch.myInstance = new MayLibSearch(textArea, Father);
		
		return MayLibSearch.myInstance;	
	}
	
	

	private void setmyFontChild()
	{		
		setFontComponents(this.getContentPane());	
	}
	
	
	private void setFontComponents(Container container)
	{
		for (Component c : container.getComponents())
		{
			if (c instanceof JLabel)
				((JLabel)c).setFont(Boldfont);
			else if (c instanceof JButton)
				((JButton)c).setFont(Boldfont);
			else if (c instanceof JCheckBox)
				((JCheckBox)c).setFont(Boldfont);
			else if (c instanceof JTextField)
				((JTextField)c).setFont(Plainfont);
			else if (c instanceof JPanel)
			{
				// Prfen, ob das Panel einen TitledBorder hat, dessen Text
				// dann ebenfalls bersetzt werden muss.
				Border b = ((JPanel)c).getBorder();
				if (b instanceof TitledBorder)
					((TitledBorder)b).setTitleFont(Plainfont);
					
				// Ein Panel ist ein Container und enthlt in aller Regel Komponenten
				setFontComponents((JPanel)c);
			}
			
		}
	}

	
	private void initializeComponents()
	{
		this.setTitle("Search the abstract for a word or a part of a word/phrase");
		this.setSize(450, 100);
		this.setLayout(null);
		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(this);
		this.setModalityType(DEFAULT_MODALITY_TYPE);
		
		lbl1 = new JLabel("Search abstract for:");
		lbl1.setBounds(5, 5, 150, 25);
		this.add(lbl1);
		
		tfSuche = new JTextField();
		tfSuche.setBounds(160, 5, 260, 25);
		this.add(tfSuche);
		
				
		checkGrossKlein = new JCheckBox("Case sensitivity");
		checkGrossKlein.setBounds(5, 40, 150, 25);
		this.add(checkGrossKlein);
		
		btnWeitersuchen = new JButton("Start search");
		//btnWeitersuchen.setFont(this.getContentPane().getFont().deriveFont(11.0f));
		btnWeitersuchen.setBounds(160, 40, 120, 25);
		btnWeitersuchen.setMargin(new Insets(2, 2, 2, 2));
		btnWeitersuchen.addActionListener(this);
		this.add(btnWeitersuchen);

		
		btnBeenden = new JButton("Close");
		//btnBeenden.setFont(this.getContentPane().getFont().deriveFont(11.0f));
		btnBeenden.setBounds(300,40, 120, 25);
		btnBeenden.setMargin(new Insets(2, 2, 2, 2));
		btnBeenden.addActionListener(this);
		this.add(btnBeenden);
			
				
	}
	
	
	private void initDialog()
	{
		if (this.newDialog) centerDialog();
		
		this.setBackground(Father.BackGroundColor);
		this.getContentPane().setBackground(Father.BackGroundColor);
		checkGrossKlein.setBackground(Father.BackGroundColor);
		setmyFontChild();
		
		this.setAlwaysOnTop(true);
	}
	
	private void centerDialog()
	{
		// Wenn 'owner' null ist wird der Dialog in der
		// Mitte des Desktops angezeigt, ansonsten
		// in der Mitte der angegebenen Komponente
		// (JFrame, JDialog...).
		this.setLocationRelativeTo(owner);
		this.newDialog = false;
		
	}
	
	
	private void suche()
	{
		if (tfSuche.getText().length() == 0)
			return;
		
		
		foundPos++;
		
		
		if (checkGrossKlein.isSelected())
			// Gro-Kleinschreibung beachten
			foundPos = textArea.getText().indexOf(tfSuche.getText(), foundPos);
		else
			// Gro-Kleinschreibung nicht beachten
			foundPos = textArea.getText().toLowerCase().indexOf(tfSuche.getText().toLowerCase(), foundPos);
		
		if (foundPos > -1)
		{
			// text in der TextArea markieren
			this.textArea.select(foundPos, foundPos + tfSuche.getText().length());
			
			// Durch den Fokusverlust ist der markierte Text im Hauptfenster nicht
			// mehr sichtbar. Durch das erneute Setzen des Fokus wird die Markierung
			// wieder angezeigt.
			this.textArea.requestFocusInWindow();
			tfSuche.requestFocusInWindow();	
		}
		else
		{
			String msg = "The search has reached the end of the text.\nDo you want to continue the search from the beginning?";
			// Bentzdefinierten Button Texte
			Object[] options = {"OK", "Cancel"};
			int retValue = JOptionPane.showOptionDialog(this,  msg, "Continue search?", JOptionPane.OK_CANCEL_OPTION, 
					                                    JOptionPane.QUESTION_MESSAGE, null, options, options[1]);
			
			if (retValue == 0) suche();
					
		}
		
	}
	
	
	
	public void showDialog()
	{
		initDialog();
		//this.setModalityType(DEFAULT_MODALITY_TYPE);
		this.setVisible(true);
		this.setAlwaysOnTop(true);
	}
	
	public void showDialog(Component owner)
	{
		this.owner = owner;
		showDialog();
		
	}
	
	
	@Override
	public void windowActivated(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowClosed(WindowEvent arg0)
	{
	
		// Die Instanzvariable lschen, damit der Garbage-Collector
		// die Klasse vom Heap entfernt und ber die getInstance()-Methode
		// eine neue Instanz des Dialogs erstellt wird.
		MayLibSearch.myInstance = null;
		
	}

	@Override
	public void windowClosing(WindowEvent arg0)
	{
		// Rift windowClosed() auf
		this.dispose();
		
	}

	@Override
	public void windowDeactivated(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowDeiconified(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowIconified(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowOpened(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource().equals(btnBeenden))
			windowClosing(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
		else if (e.getSource().equals(btnWeitersuchen))
		    suche();
	}

}
