/**
 * 
 */
package MayLib;


/**
 * @author Dr. Andreas May, email: may_devonian at yahoo.es
 *
 */

public class Periodical implements Comparable<Periodical>
{
	private String Name, Place, Abreviated_Name, ISSN, Language; 
	
	public static int getAttributeLength(String Attribute)
	{
		if (Attribute.equalsIgnoreCase("Name"))
			return 400;
		else if (Attribute.equalsIgnoreCase("Place"))
			return 150;
		else if (Attribute.equalsIgnoreCase("Abreviated_Name"))
			return 300;
		else if (Attribute.equalsIgnoreCase("ISSN"))
			return 20;
		else if (Attribute.equalsIgnoreCase("Language"))
			return 50;
		else
			return 1000;
				
	}

	/*Length & Tooltip in Access DB: 
	Name: 255, Name of the Periodical --  OBLIGATORY FIELD (Must be filled)!
	Place:100, Publishing Place (and eventually Publishing House)
	Abreviated_Name: 255, Abbreviated Name of the Periodical
	ISSN: 15,
	Language: 30,  Note if  the papers are published in Russian or Chinese 
	*/
	public static String getAttributeTooltip(String Attribute)
	{
		if (Attribute.equalsIgnoreCase("Name"))
			return "Name of the Periodical - OBLIGATORY FIELD (Must be filled)!";
		else if (Attribute.equalsIgnoreCase("Place"))
			return "Publishing Place (and eventually Publishing House)";
		else if (Attribute.equalsIgnoreCase("Abreviated_Name"))
			return "Abbreviated Name of the Periodical";
		else if (Attribute.equalsIgnoreCase("ISSN"))
			return "ISSN-Number of the Periodical";
		else if (Attribute.equalsIgnoreCase("Language"))
			return "Note if the papers are published in Russian or Chinese etc.";
		else
			return "";
				
	}
	
	// Now they are filling all related to "implements Comparable<Periodical>"
	// compareValue = Standardwert fr den Vergleich 
	private String compareValue;
	
	public String getCompareValue()
	{
		return this.compareValue;
	}
	
	// makeCompareValue permits to construct the value of compareValue,
	// which has the function as kind of primary key.
	// In this way it can be proven, that a given combination exists only one time.
	public static String makeCompareValue(String Name, String Place)
	{
		String s = "Error";
		
		if (Place.length() == 0)
			s = Name;
		else 
			s = Name + " (" + Place + ")";		
		
		return s;
	}
		
	public Periodical (String Name, String Place)
	{
		this.setNamePlace(Name, Place);
	}
	
	public void setNamePlace (String Name, String Place)
	{
		this.Name = Name;
		this.Place = Place;
		this.compareValue = makeCompareValue(this.Name, this.Place);
	}
	
	public String toString()
	{
		return this.compareValue;
	}

	// Implementierung Comparable-Schnittstelle um eine aufsteigende Sortierung zu erreichen.
	// compareToIgnoreCase fr die Sortierungen, da sonst die
	// Grossbuchstaben vor die Kleinbuchstaben sortiert werden.
	@Override
	public int compareTo(Periodical that)
	{
		
		// Aufsteigend sortieren
		return this.compareValue.compareToIgnoreCase(that.compareValue);
		// Absteigend absteigend
		//	return this.compareValue.compareToIgnoreCase(that.compareValue) * -1;
	}
	
	
	// Wenn equals() nicht sinnvoll berschrieben wird verhlt es sich wie ==
	// also werden Referenzen verglichen und keine Werte.
	
	@Override
	public boolean equals(Object that)
	{
		
		// Wenn auf das gleiche Objekt verwiesen wird
		if (this == that) return true;
		
		// Wenn der Argumenttyp ungltig (kein Vergleich von pfel mit Birnen)
		if (!(that instanceof Periodical)) return false;
				
		return (this.compareValue.compareTo(((Periodical)that).compareValue) == 0);
	
	}

	// Eine Klasse die 'equals() berschreibt muss auch die Methode 
	// hashCode() berschreiben.
	@Override
	public int hashCode()
	{
		
		if (this.compareValue == null)
			return 0;
		
		return this.compareValue.hashCode();
		
	}
	
	public String getName()
	{
		return this.Name;
	}
	
	public String getPlace()
	{
		return this.Place;
	}
	
	public String getAbreviated_Name()
	{
		return this.Abreviated_Name;
	}
	
	public String getISSN()
	{
		return this.ISSN;
	}
	
	public String getLanguage()
	{
		return this.Language;
	}
	
	public void setAbreviated_Name(String Abreviated_Name)
	{
		this.Abreviated_Name = Abreviated_Name;
	}
	
	public void setISSN(String ISSN)
	{
		this.ISSN = ISSN;
	}
	
	public void setLanguage(String Language)
	{
		this.Language = Language;
	}

	public String expComplete()
	{
		// Exporting all of a Periodical to a String, for a Report

		StringBuilder sb = new StringBuilder();
		sb.append(Name).append(". ");

		if (Place.length() > 0)
			sb.append("Place: ").append(Place).append("; ");

		if (Abreviated_Name.length() > 0)
			sb.append("Abreviated name: ").append(Abreviated_Name).append("; ");

		if (ISSN.length() > 0)
			sb.append("ISSN: ").append(ISSN).append("; ");

		if (Language.length() > 0)
			sb.append("Language: ").append(Language).append(".");

		return sb.toString();
	}
		
	
}
