package MayLib;

import java.awt.Component;
import java.awt.Container;
//import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.KeyboardFocusManager;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.TreeSet;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

public class PeriodicalDialog extends AbstractDialog implements ActionListener, KeyListener, WindowListener, FocusListener
{
	private static final long serialVersionUID = 1L;
	private Font Plainfont, Boldfont; 

	private JMenuBar	       menuBar;
	private JMenu	           menuFile, menuEdit, submenuSaveAs;
	private JMenuItem	       miNewPeriodical, miNewLibref, miClose, miSave, miSaveAsOther, miSaveAsUCF, miSaveAsUnknown, miQuitApp;
	
	// Menue Report
	private JMenu menuReport, submenuReportType;
	private JMenuItem  miReportAllPeriodicals, miReportSelObjects;
	private JRadioButtonMenuItem  miBasicReport, miCompleteReport;
	private ButtonGroup  bgReportType;
	//Addition to menue Report
	private JMenu	              submenuRepCharacterset;
	private JRadioButtonMenuItem  miRepUnicode, miRepDefault, miRepUnknown;
	private ButtonGroup		      bgRepCharacterset;
	
	
	private JLabel	     jlabelName, jlabelPlace, jlabelLanguage;
	private JLabel	     jlabelAbreviated_Name, jlabelISSN;
	private JTextField	 tfAbreviated_Name, tfISSN;
	private JTextField	 tfName, tfPlace, tfLanguage;
	private JButton	     btnOK, btnCancel, btnNewLibref, btnNewPeriodical, btnReport, btnQuitApp;
	
	
	private KeyboardFocusManager kbFocusManager;
	private Periodical myPeriodical;
	private MayLibTable Periodical_MayLibTable, Libref_MayLibTable;
	private MayLib_Start Father;
	private LibrefSplitPane myLibrefSplitPane;
	private String oldCompareValue;
		
	public PeriodicalDialog(Periodical myPeriodical, MayLib_Start Father, MayLibTable Periodical_MayLibTable, 
			MayLibTable Libref_MayLibTable, LibrefSplitPane myLibrefSplitPane)
	{
		this.myLibrefSplitPane = myLibrefSplitPane;
		this.myPeriodical = myPeriodical;
		this.Father = Father;
		this.Periodical_MayLibTable = Periodical_MayLibTable;
		this.Libref_MayLibTable = Libref_MayLibTable;
		// THis construction Periodical_MayLibTable -Father is necessary to be able to be called from different
		// dialog frames which are called from MayLib_Start
		this.Plainfont = this.Father.Plainfont;
		this.Boldfont = this.Father.Boldfont;
		this.setFont(Plainfont);
		initializeComponents();
	}
	

	private void setmyFontChild()
	{	
	
		this.getJMenuBar().setFont(Boldfont);		
		
		for (int i = 0; i < menuBar.getMenuCount(); i++)
		{
			menuBar.getMenu(i).setFont(Boldfont);
			setFontMenuItems(menuBar.getMenu(i));
		}
		
		setFontComponents(this.getContentPane());
	}
	
	private void setFontMenuItems(JMenu m)
	{
		for (Component c : m.getMenuComponents())
		{
			// Der Separator hat keinen Text und keine Elemente
			if (c instanceof JMenuItem)
			{
				((JMenuItem)c).setFont(Boldfont);
				if (((JMenuItem) c).getSubElements().length > 0)
					setFontMenuItems((JMenu) c);
			}
		}
	}

	private void setFontComponents(Container container)
	{
		for (Component c : container.getComponents())
		{
			if (c instanceof JLabel)
				((JLabel) c).setFont(Boldfont);
			else if (c instanceof JTextField)
				((JTextField) c).setFont(Plainfont);
			else if (c instanceof JButton)
				((JButton) c).setFont(Boldfont);
			else if (c instanceof JPanel)
			{
				// Prfen, ob das Panel einen TitledBorder hat, dessen Text
				// dann ebenfalls bersetzt werden muss.
				Border b = ((JPanel) c).getBorder();
				if (b instanceof TitledBorder)
					((TitledBorder) b).setTitleFont(Plainfont);

				// Ein Panel ist ein Container und enthlt in aller Regel
				// Komponenten
				setFontComponents((JPanel) c);
			}

		}
	}

	private void createMenu()
	{
		menuBar = new JMenuBar();
		this.setJMenuBar(menuBar);

		menuFile = this.createMenu(menuBar, "File", "menuFile", 'F');
		miSave = this.createMenuItem(menuFile, "miSave", AbstractDialog.MenuItemType.ITEM_PLAIN, this, "Save opened MayLib file (*.ucf)", null, 'S',
				"Saves the opened MayLib as MayLib file (*.ucf) under the same name");
		submenuSaveAs = this.createSubMenue(menuFile, "Save opened MayLib as...", "submenuSaveAs", 'a');
		miSaveAsUCF = this.createMenuItem(submenuSaveAs, "miSaveAsUCF", AbstractDialog.MenuItemType.ITEM_PLAIN, this, "MayLib file (*.ucf)", null, 'b',
				"Saves the opened MayLib as MayLib file (*.ucf) under another name");
		miSaveAsOther = this.createMenuItem(submenuSaveAs, "miSaveAsOther", AbstractDialog.MenuItemType.ITEM_PLAIN, this, "System default character set", null,
				'y', "Saves the opened MayLib as a file with the system default character set under another name");
		miSaveAsUnknown = this.createMenuItem(submenuSaveAs, "miSaveAsUnknown", AbstractDialog.MenuItemType.ITEM_PLAIN, this, "Unknown character set", null, 0,
				"Saves the opened MayLib as a file with the character set for opened files with an unknown character set");
		menuFile.addSeparator();
		miClose = this.createMenuItem(menuFile, null, AbstractDialog.MenuItemType.ITEM_PLAIN, this, "Close", null, 'C', "Closes this window");
		miQuitApp = this.createMenuItem(menuFile, null, AbstractDialog.MenuItemType.ITEM_PLAIN, this, "Quit application", null, 'Q', "Quit application");

		menuEdit = this.createMenu(menuBar, "Edit", null, 'E');
		miNewLibref = this.createMenuItem(menuEdit, "miNewLibref", AbstractDialog.MenuItemType.ITEM_PLAIN, this, "New Literature reference", null, 'L',
				"Opens window to create a new literature reference");
		miNewPeriodical = this.createMenuItem(menuEdit, "miNewPeriodical", AbstractDialog.MenuItemType.ITEM_PLAIN, this, "New Periodical", null, 'P',
				"Opens window to create a new periodical");

		menuReport = this.createMenu(menuBar, "Report", "menuReport", 'R');
		miReportAllPeriodicals = this.createMenuItem(menuReport, "miReportAllPeriodicals", AbstractDialog.MenuItemType.ITEM_PLAIN, this,
				"Report all periodicals", null, 'p', "Creates a report of all periodicals");
		miReportSelObjects = this.createMenuItem(menuReport, "miReportSelObjects", AbstractDialog.MenuItemType.ITEM_PLAIN, this, "Report visible object",
				null, 'v', "Creates a report of the visible object");
		menuReport.addSeparator();
		submenuReportType = this.createSubMenue(menuReport, "Type of Report", "submenuReportType", 'T');
		miBasicReport = (JRadioButtonMenuItem) this.createMenuItem(submenuReportType, "miBasicReport", AbstractDialog.MenuItemType.ITEM_RADIO, this,
				"Basic report", null, 'B', "A short report with the most relevant information");
		miCompleteReport = (JRadioButtonMenuItem) this.createMenuItem(submenuReportType, "miCompleteReport", AbstractDialog.MenuItemType.ITEM_RADIO, this,
				"Complete report", null, 'C', "A complete report with all information");
		bgReportType = new ButtonGroup();
		bgReportType.add(miBasicReport);
		bgReportType.add(miCompleteReport);
		//Addition to menue Report
		submenuRepCharacterset = this.createSubMenue(menuReport, "Characterset of the report",  "submenuRepCharacterset", 'h');
		miRepUnicode = (JRadioButtonMenuItem)this.createMenuItem(submenuRepCharacterset, "miRepUnicode", AbstractDialog.MenuItemType.ITEM_RADIO, this, "Unicode (*.RPU)", null, 'U', "The resulting report will be written with Unicode character set. The default file extension is *.RPU");
		miRepDefault = (JRadioButtonMenuItem)this.createMenuItem(submenuRepCharacterset, "miRepDefault", AbstractDialog.MenuItemType.ITEM_RADIO, this, "System default character set (*.RPT)", null, 'y', "The resulting report will be written with the default system character set. The default file extension is *.RPT");
		miRepUnknown = (JRadioButtonMenuItem)this.createMenuItem(submenuRepCharacterset, "miRepUnknown", AbstractDialog.MenuItemType.ITEM_RADIO, this, "Unknown character set (*.RPT)", null, 0, "The resulting report will be written with the character set used by the last file imported with UNKNOWN character set. The default file extension is *.RPT");
		bgRepCharacterset = new ButtonGroup();
		bgRepCharacterset.add(miRepUnicode);
		bgRepCharacterset.add(miRepDefault);
		bgRepCharacterset.add(miRepUnknown);	
		
	}
	
	private void fillFields()
	{
		tfName.setText(myPeriodical.getName());
		tfName.setCaretPosition(0);
		tfPlace.setText(myPeriodical.getPlace());
		tfPlace.setCaretPosition(0);
		tfAbreviated_Name.setText(myPeriodical.getAbreviated_Name());
		tfAbreviated_Name.setCaretPosition(0);
		tfISSN.setText(myPeriodical.getISSN());
		tfISSN.setCaretPosition(0);
		tfLanguage.setText(myPeriodical.getLanguage());
		tfLanguage.setCaretPosition(0);
	}
	

	@Override
	public void initializeComponents()
	{
		this.setTitle("Periodical");
		this.setLayout(null);
		this.setModalityType(DEFAULT_MODALITY_TYPE);
		this.setBounds(10, 10, 800, 300);
		this.setDefaultCloseOperation(AbstractDialog.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(this);
		
		createMenu();
		

		jlabelName = new JLabel("Name");
		jlabelName.setBounds(10, 10, 80, 25);
		jlabelName.setName("Name");
		this.add(jlabelName);

		tfName = new JTextField();
		tfName.setBounds(90, 10, 690, 25);
		tfName.setToolTipText(myPeriodical.getAttributeTooltip("Name"));
		tfName.setName("Name");
		tfName.addKeyListener(this);
		tfName.addFocusListener(this);
		this.add(tfName);

		jlabelPlace = new JLabel("Place");
		jlabelPlace.setBounds(10, 40, 80, 25);
		jlabelPlace.setName("Place");
		this.add(jlabelPlace);

		tfPlace = new JTextField();
		tfPlace.setBounds(90, 40, 690, 25);
		tfPlace.setName("Place");
		tfPlace.setToolTipText(myPeriodical.getAttributeTooltip("Place"));
		tfPlace.addKeyListener(this);
		tfPlace.addFocusListener(this);
		this.add(tfPlace);
		
		jlabelAbreviated_Name = new JLabel("Abreviated Name");
		jlabelAbreviated_Name.setBounds(10, 70, 140, 25);
		jlabelAbreviated_Name.setName("Abreviated_Name");
		this.add(jlabelAbreviated_Name);

		tfAbreviated_Name = new JTextField();
		tfAbreviated_Name.setBounds(140, 70, 640, 25);
		tfAbreviated_Name.setName("Abreviated_Name");
		tfAbreviated_Name.setToolTipText(myPeriodical.getAttributeTooltip("Abreviated_Name"));
		tfAbreviated_Name.addKeyListener(this);
		tfAbreviated_Name.addFocusListener(this);
		this.add(tfAbreviated_Name);

		jlabelISSN = new JLabel("ISSN");
		jlabelISSN.setBounds(10, 100, 80, 25);
		jlabelISSN.setName("ISSN");
		this.add(jlabelISSN);

		tfISSN = new JTextField();
		tfISSN.setBounds(90, 100, 345, 25);
		tfISSN.setName("ISSN");
		tfISSN.setToolTipText(myPeriodical.getAttributeTooltip("ISSN"));
		tfISSN.addKeyListener(this);
		tfISSN.addFocusListener(this);
		this.add(tfISSN);
		
		jlabelLanguage = new JLabel("Language");
		jlabelLanguage.setBounds(10, 130, 80, 25);
		jlabelLanguage.setName("Language");
		this.add(jlabelLanguage);

		tfLanguage = new JTextField();
		tfLanguage.setBounds(90, 130, 345, 25);
		tfLanguage.setName("Language");
		tfLanguage.setToolTipText(myPeriodical.getAttributeTooltip("Language"));
		tfLanguage.addKeyListener(this);
		tfLanguage.addFocusListener(this);
		this.add(tfLanguage);
		
	    //Buttons
		btnNewLibref = new JButton("New Literature");
		btnNewLibref.setBounds(10, 180, 120, 25);
		btnNewLibref.setMargin(new Insets(2, 2, 2, 2));
		btnNewLibref.setName("New Literature");
		btnNewLibref.addKeyListener(this);
		btnNewLibref.addActionListener(this);
		this.add(btnNewLibref);

		btnNewPeriodical = new JButton("New Periodical");
		btnNewPeriodical.setName("New Periodical");
		btnNewPeriodical.setBounds(140, 180, 120, 25);
		btnNewPeriodical.setMargin(new Insets(2, 2, 2, 2));
		btnNewPeriodical.addActionListener(this);
		btnNewPeriodical.addKeyListener(this);
		this.add(btnNewPeriodical);

		btnReport = new JButton("Report");
		btnReport.setName("Report");
		btnReport.setBounds(270, 180, 120, 25);
		btnReport.setMargin(new Insets(2, 2, 2, 2));
		btnReport.addActionListener(this);
		btnReport.addKeyListener(this);
		this.add(btnReport);
		
		btnOK = new JButton("Save & Close");
		btnOK.setBounds(400, 180, 120, 25);
		btnOK.setMargin(new Insets(2, 2, 2, 2));
		btnOK.setName("Save");
		btnOK.addActionListener(this);
		btnOK.addKeyListener(this);
		this.add(btnOK);

		btnCancel = new JButton("Cancel & Close");
		btnCancel.setName("Cancel");
		btnCancel.setBounds(530, 180, 120, 25);
		btnCancel.setMargin(new Insets(2, 2, 2, 2));
		btnCancel.addActionListener(this);
		btnCancel.addKeyListener(this);
		this.add(btnCancel);

		btnQuitApp = new JButton("Quit application");
		btnQuitApp.setName("Quit");
		btnQuitApp.setBounds(660, 180, 120, 25);
		btnQuitApp.setMargin(new Insets(2, 2, 2, 2));
		btnQuitApp.addActionListener(this);
		btnQuitApp.addKeyListener(this);
		this.add(btnQuitApp);
		
	}

	private boolean saveEntry()
	{
		if (!Father.hasChanged) return true;
		
		// If all fields are empty, we do not construct a new element
		if ((cleanEntry(tfName.getText()).length() > 0) || (cleanEntry(tfPlace.getText()).length() > 0)
				|| (cleanEntry(tfAbreviated_Name.getText()).length() > 0) || (cleanEntry(tfISSN.getText()).length() > 0)
				|| (cleanEntry(tfLanguage.getText()).length() > 0))
		{
			// As a first step, we have to clean the old comparevalue
			this.oldCompareValue = null;

			if (!eingabeOK())
				return false;

			// Now myPeriodical contains the actual data
			// and !Father.hasChanged = TRUE
			// We actualize the data in the other frames
			
			//We eliminate the old entry
			if (this.oldCompareValue != null)
				Father.PeriodicalTree.remove(oldCompareValue);
			
			Father.PeriodicalTree.put(myPeriodical.getCompareValue(), myPeriodical);
			
			if (Periodical_MayLibTable != null)
				Periodical_MayLibTable.MyTableTreeSet.add(myPeriodical);
		}

		if (!Father.hasChanged) return true;
		
		String fileExtension = "UCF", Characterset = "UTF-16";
		String Dateiname = Father.MayLibFile;

		return Father.saveMayLib(Dateiname, fileExtension, Characterset);
	}
	
	
	private String cleanEntry(String EntryString)
	{
		return EntryString.replace(";", ",").replace("\n", " ").replace("\r", " ").replace("\t", " ").trim();
	}
	
	private boolean eingabeOK()
	{	
		// if nothing has been changed we do nothing
		if (!Father.hasChanged) return true;
		
		if (cleanEntry(tfName.getText()).length() == 0)
		{
			JOptionPane.showMessageDialog(this, "The field \"Name\" must be filled", "Missing data", JOptionPane.ERROR_MESSAGE);
			tfName.requestFocusInWindow();
			return false;
		}
				
		//Clean and shorten if necessary the entries
		String S_Name = cleanEntry(tfName.getText());
		if (S_Name.length() > Periodical.getAttributeLength("Name"))
			S_Name =S_Name.substring(0, Periodical.getAttributeLength("Name"));
		
		String S_Place = cleanEntry(tfPlace.getText());
		if (S_Place.length() > Periodical.getAttributeLength("Place"))
			S_Place = S_Place.substring(0, Periodical.getAttributeLength("Place"));
		
		String S_Abreviated_Name = cleanEntry(tfAbreviated_Name.getText());
		if (S_Abreviated_Name.length() > Periodical.getAttributeLength("Abreviated_Name"))
			S_Abreviated_Name = S_Abreviated_Name.substring(0, Periodical.getAttributeLength("Abreviated_Name"));
		
		String S_ISSN = cleanEntry(tfISSN.getText());
		if (S_ISSN.length() > Periodical.getAttributeLength("ISSN"))
			S_ISSN = S_ISSN.substring(0, Periodical.getAttributeLength("ISSN"));
		
		String S_Language = cleanEntry(tfLanguage.getText());
		if (S_Language.length() > Periodical.getAttributeLength("Language"))
			S_Language =S_Language.substring(0, Periodical.getAttributeLength("Language"));
		
		//Prove on duplicates
		String Teststring = Periodical.makeCompareValue(S_Name, S_Place);
		
		if (Father.PeriodicalTree.containsKey(Teststring))
		{
			if (myPeriodical == null)
			{

				JOptionPane.showMessageDialog(this, "A periodical with this combination \"Name\" and \"Place\" already exists.", "Conflict with existing data",
						JOptionPane.ERROR_MESSAGE);
				tfName.requestFocusInWindow();
				return false;
			}

			else
			{
				Periodical ParentPeriodical = Father.PeriodicalTree.get(Teststring);

				if (ParentPeriodical != myPeriodical)
				{
					JOptionPane.showMessageDialog(this, "Another periodical with this combination \"Name\" and \"Place\" already exists.",
							"Conflict with existing data", JOptionPane.ERROR_MESSAGE);
					tfName.requestFocusInWindow();
					return false;
				}
			}

		}
		
		// Now we know that it is o.k.
		// We construct a new periodical
		
		// As a first step, we have to actualize the old comparevalue with the comparevalue of the old myPeriodical
		if (this.myPeriodical != null)
			this.oldCompareValue = myPeriodical.getCompareValue();
		else
			this.oldCompareValue = null;

		
		if (myPeriodical == null)
		{
			myPeriodical = new Periodical(S_Name, S_Place);
		}
		else
		{
			myPeriodical.setNamePlace(S_Name, S_Place);
		}
		
		myPeriodical.setAbreviated_Name(S_Abreviated_Name);
		myPeriodical.setISSN(S_ISSN);
		myPeriodical.setLanguage(S_Language);
		
		// assure that the change is notified
		Father.hasChanged = true; 
		Father.needpopulateCboPeriod = true;
		miReportSelObjects.setEnabled(true);
		btnReport.setEnabled(true);
		fillFields();
		
		return true;
	}

	
	private boolean queryExit()
	{
			
		// Benutzerdefinierten Button Text
		Object[] options = {"Yes", "No", "Cancel"};
		
		if (!Father.hasChanged) return true;
		
		int retValue = JOptionPane.showOptionDialog(this, "The literature data have been changed.\nDo you want to save the changes?", "Save the changes?", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[2]);
		
		// Abbrechen ?
		if (retValue == 2) return false;
		
		// Ja - Speichern ?
		if (retValue == 0)
		{
			boolean result = saveEntry();
			
			if (result==true)
			{
				JOptionPane.showMessageDialog(null, "Changes in opened MayLib database saved (File: " + Father.lastWrittenFile +").", "Data saved",
						JOptionPane.INFORMATION_MESSAGE);	
			}
			
			return result;
		}
		
		// Nein - Nicht speichern
		return true;			
	}
	
	
	@Override
	public void initDialog()
	{
		this.getContentPane().setBackground(Father.BackGroundColor);
		
		// Windowsize
		if (Father.PeriodicalDialog_Width > 0 && Father.PeriodicalDialog_Height > 0)
			this.setBounds(Father.PeriodicalDialog_X, Father.PeriodicalDialog_Y, Father.PeriodicalDialog_Width, Father.PeriodicalDialog_Height);
		else
		{
			if (Father.MaximizeWindow.equalsIgnoreCase("all"))
				makeFrameFullSize();
		}
		
		kbFocusManager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
		
		setmyFontChild();
		
		miSave.setEnabled(Father.hasChanged);

		// activate /deactivate Report Menu
		menuReport.setEnabled(true);
		submenuReportType.setEnabled(true);
		miReportAllPeriodicals.setEnabled(Father.PeriodicalTree.size() > 0);
		miReportSelObjects.setEnabled(myPeriodical != null);
		miBasicReport.setEnabled(true);
		miCompleteReport.setEnabled(true);
	
		miBasicReport.setSelected(true);
		//miRepUnicode.setSelected(true);
		
		// Defines charset of report
		if (Father.strexecRepCharset.equalsIgnoreCase("Default"))
		{
			this.miRepDefault.setSelected(true);
		}
		else if (Father.strexecRepCharset.equalsIgnoreCase("Unknown"))
		{
			this.miRepUnknown.setSelected(true);
		}
		else
		{
			this.miRepUnicode.setSelected(true);
		}

		
		btnReport.setEnabled(myPeriodical != null);
		
		if (myPeriodical != null) 
		{
			fillFields();
		}
	}


	
	@Override
	public void focusGained(FocusEvent e)
	{
		// TODO Auto-generated method stub
	}

	@Override
	public void focusLost(FocusEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowActivated(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowClosed(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowClosing(WindowEvent arg0)
	{
		// berprfen, ob nderungen noch gespeichert werden mssen
		if (queryExit())
			this.execSizeDispose();
		else
			tfName.requestFocusInWindow();
		
	}

	@Override
	public void windowDeactivated(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowDeiconified(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowIconified(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowOpened(WindowEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyPressed(KeyEvent e)
	{
		
		if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{

			if (e.getSource().equals(tfName) && cleanEntry(tfName.getText()).length() == 0)
				return;

			else if (e.getSource().equals(btnCancel))
				windowClosing(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));

			else if (e.getSource().equals(btnOK))
			{
				if (Father.hasChanged && !saveEntry())
					return;
				windowClosing(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
			}

			else if (e.getSource().equals(btnNewPeriodical))
				clickmiNewPeriodical();
			else if (e.getSource().equals(btnNewLibref))
				clickmiNewLibref();

			else if (e.getSource().equals(btnReport))
			{
				JOptionPane.showMessageDialog(null, "Feature btnReport not yet implemented ", "Not implemented", JOptionPane.INFORMATION_MESSAGE);
			}

			else if (e.getSource().equals(btnQuitApp))
			{
				clickmiQuitApp();
			}
			
			else if (e.getSource().equals(btnReport))
			{	
				TreeSet<Object> ReportTreeSet = new TreeSet<Object>();
				ReportTreeSet.add(this.myPeriodical);
				
				Father.createReport(ReportTreeSet, "Periodical", this.getTitle(), "without", "complete", "RPU", "UTF-16");
			}		

			else

				kbFocusManager.focusNextComponent();
		}
		
	}

	@Override
	public void keyReleased(KeyEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyTyped(KeyEvent e)
	{
		if (e.getSource() instanceof JTextField)
		{
	
			if (((JTextField)e.getSource()).getText().length() >= Periodical.getAttributeLength(((JTextField)e.getSource()).getName()))
			{
					Toolkit.getDefaultToolkit().beep();
					e.consume();
					return;
			}
		
		Father.hasChanged = true;
		miSave.setEnabled(true);
		}
	}

	@Override
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource().equals(btnCancel))
			windowClosing(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
		
		else if (e.getSource().equals(btnOK))
		{
			if (Father.hasChanged &&  !saveEntry()) return;
			windowClosing(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
		}
		
		else if (e.getSource().equals(btnNewPeriodical))
			clickmiNewPeriodical();
		else if (e.getSource().equals(btnNewLibref))
			clickmiNewLibref();

		else if (e.getSource().equals(btnReport))
		{	
			TreeSet<Object> ReportTreeSet = new TreeSet<Object>();
			ReportTreeSet.add(this.myPeriodical);
			
			Father.createReport(ReportTreeSet, "Periodical", this.getTitle(), "without", "complete", "RPU", "UTF-16");
		}		
		
		else if (e.getSource().equals(miClose))
			windowClosing(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));

		
		else if (e.getSource().equals(miQuitApp))
		{
			clickmiQuitApp();
		}
		
		else if (e.getSource().equals(btnQuitApp))
		{
			clickmiQuitApp();
		}
		
		else if (e.getSource().equals(miNewPeriodical))
			clickmiNewPeriodical();
		else if (e.getSource().equals(miNewLibref))
			clickmiNewLibref();
		

		else if (e.getSource().equals(miSave))	
		{
			//String fileExtension = "UCF", Characterset = "UTF-16";
			//String Dateiname = Father.MayLibFile;
			
			if (saveEntry())
			{JOptionPane.showMessageDialog(null, "Changes in opened MayLib database saved (File: " + Father.lastWrittenFile +").", "Data saved",
					JOptionPane.INFORMATION_MESSAGE);
			}
		}

		else if (e.getSource().equals(miSaveAsUCF))	
		{
			String fileExtension = "UCF", Characterset = "UTF-16";
			String Dateiname = null;
			
			if (saveEntry() && Father.saveMayLib(Dateiname, fileExtension, Characterset))
			{JOptionPane.showMessageDialog(null, "Opened MayLib database saved sucessfully in the file " + Father.lastWrittenFile, "Data saved",
					JOptionPane.INFORMATION_MESSAGE);
			}
		}

		else if (e.getSource().equals(miSaveAsOther))	
		{
			String fileExtension = "TXT", Characterset = Father.Default_Charset;
			String Dateiname = null;
			
			if (saveEntry() && Father.saveMayLib(Dateiname, fileExtension, Characterset))
			{JOptionPane.showMessageDialog(null, "Opened MayLib database exported sucessfully to the file " + Father.lastWrittenFile, "Data saved",
					JOptionPane.INFORMATION_MESSAGE);
			}
		}
		
		else if (e.getSource().equals(miSaveAsUnknown))	
		{
			String fileExtension = "TXT", Characterset = "UNKNOWN";
			String Dateiname = null;
			
			if (saveEntry() && Father.saveMayLib(Dateiname, fileExtension, Characterset))
			{JOptionPane.showMessageDialog(null, "Opened MayLib database exported sucessfully to the file " + Father.lastWrittenFile, "Data saved",
					JOptionPane.INFORMATION_MESSAGE);
			}
		}
		
		// Reports
		else if ( e.getSource().equals(miReportAllPeriodicals)
				|| e.getSource().equals(miReportSelObjects))	
		{
			/* Possible values of createReport(TreeSet<Object> ReportTreeSet, String ObjectType, String Headertext, String Abstract, String ReportType, String fileExtension, String Characterset)
			
			 * String ObjectType = Libref, Periodical
			 * String Headertext = null or a prepared text terminating without linefeed (linefeed will be added in the method createReport
			 * String Abstract  = without, 1000, full. Will be ignored, if ObjectType = Periodical
			 * String ReportType = basic, complete 
			 * String fileExtension = RPU, RPT
			 * String Characterset	= "UTF-16", Default_Charset, "UNKNOWN"	
		

			reportAllPeriodicals(String ReportType, String fileExtension, String Characterset)
			*/
			
			String Abstract ="without", ReportType = "basic", fileExtension ="RPT", Characterset =  Father.Default_Charset;
				
			if (miBasicReport.isSelected())
				ReportType = "basic";
			else if (miCompleteReport.isSelected())
				ReportType = "complete";
			else 
				ReportType = "complete";
			
			if (miRepUnicode.isSelected())
			{
				fileExtension = "RPU";
				Characterset = "UTF-16";
				Father.execRepCharset("Unicode");
			}
			else if (miRepDefault.isSelected())
			{
				fileExtension ="RPT";
				Characterset =  Father.Default_Charset;
				Father.execRepCharset("Default");
			}
			else if (miRepUnknown.isSelected())
			{
				fileExtension ="RPT";
				Characterset =  "UNKNOWN";
				Father.execRepCharset("Unknown");
			}
			
			if (e.getSource().equals(miReportAllPeriodicals))
				Father.reportAllPeriodicals(ReportType, fileExtension, Characterset);
			else 
			{	
				TreeSet<Object> ReportTreeSet = new TreeSet<Object>();
				ReportTreeSet.add(this.myPeriodical);
				
				Father.createReport(ReportTreeSet, "Periodical", this.getTitle(), Abstract, ReportType, fileExtension, Characterset);
			}			
					
		}
			
		miSave.setEnabled(Father.hasChanged);
	}
	
	private void clickmiNewPeriodical()
	{
		if (!saveEntry()) return;
		
		// now we clean up
			
		this.tfAbreviated_Name.setText("");
		this.tfISSN.setText("");
		this.tfLanguage.setText("");
		this.tfName.setText("");
		this.tfPlace.setText("");

		myPeriodical = null;
		oldCompareValue = null;

		miReportSelObjects.setEnabled(false);
		btnReport.setEnabled(false);
	}
	
	private void clickmiNewLibref()
	{
		if (!saveEntry()) return;
		LibrefDialog dlg = new LibrefDialog(null, this.Father, this.Periodical_MayLibTable, this.Libref_MayLibTable, this.myLibrefSplitPane);
		dlg.showDialog(this);
	}
	
	private void clickmiQuitApp()
	{
		// Prove if changes must be saved
		if (queryExit())
		{
		
			try
			{
				this.execSizeDispose();

				if (this.Periodical_MayLibTable != null)
				{
					this.Periodical_MayLibTable.execSizeDispose();
				}

				if (this.Libref_MayLibTable != null)
				{
					this.Libref_MayLibTable.execSizeDispose();
				}
				
				if (this.myLibrefSplitPane != null)
				{
					this.myLibrefSplitPane.execSizeDispose();
				}

				this.Father.dispose();
			}
			catch (Exception e)
			{
			}

		}
		else
			tfName.requestFocusInWindow();
	}

	protected void execSizeDispose()
	{
		Father.PeriodicalDialog_X = this.getX();
		Father.PeriodicalDialog_Y = this.getY();
		Father.PeriodicalDialog_Width = this.getWidth();
		Father.PeriodicalDialog_Height = this.getHeight();
		this.dispose();
	}
}
