package MayLib;

import java.awt.*;
import java.awt.event.*;
//import java.awt.geom.RoundRectangle2D;

import javax.swing.*;

//A.May: Verschiedene Methoden auskommentiert, da sie nicht mit Java 1.6 kompatibel sind.
public class ProgressWindow extends JDialog implements Runnable, ComponentListener, MouseListener, MouseMotionListener, AWTEventListener
{
	private static final long serialVersionUID = 1L;
	private int thisWidth = 400;
	private int thisHeight = 60;
	private Color progressColor = SystemColor.activeCaption;
	private boolean showPercentage = true;
	private int progressOrientation = JProgressBar.HORIZONTAL;
	//private float thisOpacity = 1.0f;
	
	private Component owner;

	private JLabel lblStatustext;
	private JProgressBar progressBar;
	private JButton btnCancel;
	
	private Point p1, p2;
	
	private boolean cancelProcess = false;
	
	/**
	 * 
	 * @param owner
	 * Eigentmer des Fensters oder 'null' fr Desktop.
	 */
	public ProgressWindow(Component owner)
	{
		this.owner = owner;
		initializeComponents();
	}
	
	/**
	 * 
	 * @param owner
	 *  Eigentmer des Fensters oder 'null' fr Desktop.
	 * @param width
	 * Breite des Fensters.
	 * @param height
	 * Hhe des Fensters.
	 */
	public ProgressWindow(Component owner, int width, int height)
	{
		this(owner);
		thisWidth = width;
		thisHeight = height;
	}
	
	private void initializeComponents()
	{
		this.setLayout(new BorderLayout());
		//this.setAlwaysOnTop(true);
		this.setModalityType(DEFAULT_MODALITY_TYPE);
		this.addComponentListener(this);
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		// Keine Grssennderung
		this.setResizable(false);
		// Anzeige ohne Titelleiste
		this.setUndecorated(true);
		
		
		// AWTEventListener implementieren um Tastatureingaben abzufangen,
		// egal welche Komponente gerade den Fokus hat:
		//z.B. die Escape-Taste um den Dialog abzubrechen.
		Toolkit.getDefaultToolkit().addAWTEventListener(this, AWTEvent.KEY_EVENT_MASK);
		
		progressBar = new JProgressBar(progressOrientation);
		progressBar.setBorderPainted(showPercentage);
		
		// Farbe fr die Fortschrittanzeige
		progressBar.setForeground(progressColor);
		// Prozentsatz anzeigen
		progressBar.setStringPainted(true);
			
		this.add(progressBar, BorderLayout.CENTER);
		
		btnCancel = new JButton("X");
		btnCancel.setBounds(thisWidth - 16, 2, 20, 14);
		btnCancel.setFont(btnCancel.getFont().deriveFont(10.0f));
		btnCancel.setMargin(new Insets(0,0,0,0));
		btnCancel.setToolTipText("Abbrechen");
		btnCancel.setBackground(Color.RED);
		btnCancel.setForeground(Color.WHITE);
		btnCancel.setFocusPainted(false);
		btnCancel.setRolloverEnabled(false);
		//btnCancel.setBorderPainted(false);
		// Fr den Button einen MouseListener und keinen ActionListener verwenden damit nicht
		// durch bettigen der Eingabetaste irrtmlich das Programm beendet wird.
		btnCancel.addMouseListener(this);
		progressBar.add(btnCancel);
		
		lblStatustext = new JLabel();
		lblStatustext.setPreferredSize(new Dimension(this.getWidth(), thisHeight / 3));
		lblStatustext.setBorder(BorderFactory.createLineBorder(SystemColor.activeCaption));
		lblStatustext.setHorizontalAlignment(JLabel.CENTER);
		this.add(lblStatustext, BorderLayout.PAGE_END);
		
		// setSize() lst das das componentResized-Ereignis aus.
		// Dort wirtd der Cancel positioniert. 
		// Deswegen setSize() erst ganz zum Schluss aufrufen.
		this.setSize(new Dimension(thisWidth, thisHeight));
		
	}

	/**
	 * Zeigt den angegebenen Statustext an. 
	 * @param msg
	 * Statustext
	 */
	public void setMessage(String msg)
	{
		if (msg.length() > 0)
		{
			lblStatustext.setText(msg);
			lblStatustext.setVisible(true);
		}
		else
			lblStatustext.setVisible(false);
	}
	
	/**
	 * Gibt den aktuellen Statustext zurck.
	 * @return
	 */
	public String getMessage()
	{
		return lblStatustext.getText();
	}
	
	/**
	 * Setzt den aktuellen in der Fortschrittsanzeige.
	 * @param value
	 */
	public void setValue(int value)
	{
		progressBar.setValue(value);
	}
	
	public int getValue()
	{
		return progressBar.getValue();
	}
	
	/**
	 * Setzt den Maimalwert der Fortschrittsanzeige.
	 * Entspricht 100 %. 
	 * @param value
	 */
	public void setMaxValue(int value)
	{
		progressBar.setMaximum(value);
	}
	
	/**
	 * Setzt die Farbe der Fortschrittsanzeige.
	 * @param color
	 */
	public void setProgressbarColor(Color color)
	{
		progressColor = color;
		progressBar.setForeground(color);
	}
	
	/**
	 * Gibt an, ob der prozentuale Wert innerhalb der Fortschrittsanzeige
	 * angezeigt werden soll.
	 * @param value
	 */
	public void showPercentage(boolean value)
	{
		showPercentage = value;
		progressBar.setStringPainted(value);
	}
	
	
	public void setIndeterminate(boolean value)
	{
		progressBar.setIndeterminate(value);
	}
	
	/**
	 * Benutzerdefinerter Text in der Fortschrittsanzeige erstellen
	 * @param s
	 */
	public void setString(String s)
	{
		 progressBar.setString(s);
		 showPercentage(true);
	}
	
	public JProgressBar getProgressbar()
	{
		return progressBar;
	}
	
	public JLabel getStatusLabel()
	{
		return lblStatustext;
	}
	
	private void centerOnParent(Component owner)
	{
		this.setLocationRelativeTo(owner);
	}
	
//	/**
//	 * Setzt die Transparenz des Fensters.
//	 * @param value
//	 * Gltige Werte: 0 - 100.
//	 */
	// Auskommentiert, da es nicht mit Java 1.6 kompatibel ist
//	public void setTransparency(int  value)
//	{
//		thisOpacity = (float) (1 - (value / 100.0));
//		this.setOpacity(thisOpacity);
//	}
	
	public void setWidth(int  value)
	{
		thisWidth = value;
		this.setSize(thisWidth, thisHeight);
	}
	
	public void setHeight(int  value)
	{
		thisHeight = value;
		this.setSize(thisWidth, thisHeight);
	}
	
	public boolean cancelRequest()
	{
		return this.cancelProcess;
	}
	
	public void setCancelRequest(boolean b)
	{
		this.cancelProcess = b;
	}
	
	/**
	 * Zeigt die Fortschrittsanzeige an.
	 */
	private void Show()
	{
		centerOnParent(owner);
		this.setVisible(true);

	}

	
	/**
	 * Schliet die Fortschrittsanzeige.
	 */
	public void close()
	{
		this.dispose();
	}
	
	@Override
	public void run()
	{
		Show();
	}

	@Override
	public void componentHidden(ComponentEvent e)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void componentMoved(ComponentEvent e)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void componentResized(ComponentEvent e)
	{
		if (e.getSource().equals(this))
		{
			// Auskommentiert, da es nicht mit Java 1.6 kompatibel ist
			//this.setShape(new RoundRectangle2D.Float(0, 0, this.getWidth(), this.getHeight(), 5, 5));
			btnCancel.setLocation(thisWidth - btnCancel.getWidth()- 2, 2);
		}
	}

	@Override
	public void componentShown(ComponentEvent e)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseDragged(MouseEvent e)
	{
		// x und y Position des Mauszeigers ermitteln
		p2 = e.getPoint();
		// Obere Linke Ecke (x,y) des Fensters ermitteln
		Point pLocation = this.getLocation();
		// bersetzen der x und y Koordinaten entlang der
		// X- und Y-Achse
		pLocation.translate(p2.x - p1.x, p2.y - p1.y);
		// Neue Position des Fensters etzen
		this.setLocation(pLocation);
	}

	@Override
	public void mouseMoved(MouseEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseClicked(MouseEvent e)
	{
		if (e.getSource().equals(btnCancel))
			cancelProcess = true;
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent arg0)
	{
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent e)
	{
		// x und y Position des Mauszeigers ermitteln
		p1 = e.getPoint();
	}

	@Override
	public void mouseReleased(MouseEvent e)
	{
		
		// TODO Auto-generated method stub
		
	}

	@Override
	public void eventDispatched(AWTEvent event)
	{
		// Nur wenn es sich bei dem AWTEvent um ein KeyEvent handelt
		if (event instanceof KeyEvent)
		{
			KeyEvent e = (KeyEvent)event;
			if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
				this.setCancelRequest(true);
		}
		
	}
	
}
